@@ -1,12 +1,10 @@
-#Copyright barry king <barry@wyrdwright.com> and released under the GPL.
-#See http://www.gnu.org/licenses/gpl.html#TOC1 for details
 use 5.006;
 use strict;
 use warnings;
 no warnings qw(uninitialized);
 
 package Apache::Wyrd::Services::SearchParser;
-our $VERSION = '0.91';
+our $VERSION = '0.92';
 
 =pod
 
@@ -31,13 +29,20 @@ Apache::Wyrd::Services::SearchParser - Object for performing logical word-search
 =head1 DESCRIPTION
 
 Allows for logical parsing of a search using AND, OR, NOT and DIFF
-keywords.  Designed to work with C<Apache::Wyrd::Services::Index>, but
-can work with another search object.  Note that these keywords MUST be
-in upper-case to parse, otherwise they will be interpreted as the
-literal words.
+keywords.    Note that these keywords MUST be in upper-case to parse,
+otherwise they will be interpreted as the literal words.
 
 Any search object using the parser should implement a C<search()> method
-which does a word search against an index, returning an array of hashes.
+which does a word search against a reverse-key index, returning an array
+of hashes.
+
+Designed to work with C<Apache::Wyrd::Services::Index>, but can work
+with another search object as long as the search object returns arrays
+of hash items with unique IDs under the ID hash key.  This ID hash key
+defaults to 'id', which is the default key for an
+Apache::Wyrd::Services::Index object.  If your object uses a different
+ID key, it should return the (scalar) name of this ID key when it's
+C<key()> method is called.
 
 =head1 METHODS
 
@@ -48,8 +53,8 @@ I<(format: (returns) name (arguments after self))>
 =item (Apache::Wyrd::Services::SearchParser) C<new> (objectref)
 
 Create a new parser object.  The search object using the parser should
-pass itself as the argument, as C<parse> will call it's C<search> and
-C<key> methods.
+pass a reference to itself as the argument, as C<parse> will call it's
+C<search> and C<key> methods.
 
 =cut
 
@@ -80,10 +85,17 @@ and joining the results based on the logical operators.
 
 sub parse {
 	my ($self, $phrase, @options) = @_;
+
 	#remove leading bogus operators
 	$phrase =~ s/^\s*(AND|OR|DIFF)\s*//;
-	#change leading NOTs to -
+
+	#change leading NOTs to negative searches
+	#WARNING: This is only for compatibility with search engines which
+	#understand that a leading '-' in a search term means "return every
+	#document that does not match this term".  Apache::Wyrds::Services::Index
+	#is one such engine.
 	$phrase =~ s/^\s*(NOT)\s*/-/;
+
 	my $result = $self->recursive_parse($phrase, @options);
 	return @{$self->{'hash'}->{$result}};
 }
@@ -91,25 +103,25 @@ sub parse {
 sub recursive_parse {
 	my ($self, $phrase, @options) = @_;
 	my ($matched) = (1);
-	while ($matched) {#first deal with parentheticals
+	while ($matched) {#first deal with parentheticals using a non-greedy regexp.
 		$matched = $phrase =~ s/\(([^\(]*?)\)/$self->recursive_parse($1)/e;
 	}
 	$matched = 1;
-	while ($matched) {#then deal with nots
-		$matched = $phrase =~ s/(\S+)\s+NOT\s+(\S+)/$self->negation($self->recursive_parse($1),$self->recursive_parse($2))/e;
-	}
-	$matched = 1;
 	while ($matched) {#then deal with ands
 		$matched = $phrase =~ s/(\S+)\s+AND\s+(\S+)/$self->intersection($self->recursive_parse($1),$self->recursive_parse($2))/e;
 	}
 	$matched = 1;
-	while ($matched) {#then deal with diffs
-		$matched = $phrase =~ s/(\S+)\s+DIFF\s+(\S+)/$self->difference($self->recursive_parse($1),$self->recursive_parse($2))/e;
-	}
-	$matched = 1;
 	while ($matched) {#then deal with ors
 		$matched = $phrase =~ s/(\S+)\s+OR\s+(\S+)/$self->union($self->recursive_parse($1),$self->recursive_parse($2))/e;
 	}
+	$matched = 1;
+	while ($matched) {#then deal with nots
+		$matched = $phrase =~ s/(\S+)\s+NOT\s+(\S+)/$self->negation($self->recursive_parse($1),$self->recursive_parse($2))/e;
+	}
+	$matched = 1;
+	while ($matched) {#then deal with diffs
+		$matched = $phrase =~ s/(\S+)\s+DIFF\s+(\S+)/$self->difference($self->recursive_parse($1),$self->recursive_parse($2))/e;
+	}
 	return $self->get_results($phrase, @options);
 }
 
@@ -129,7 +141,7 @@ sub new_id {
 
 sub union {
 	my ($self, $a, $b) = @_;
-	my $id = new_id;
+	my $id = $self->new_id;
 	$self->{'hash'}->{$id} = $self->join_sets('u', $self->{'key'}, $self->{'hash'}->{$a}, $self->{'hash'}->{$b});
 	#use Data::Dumper;
 	#warn Dumper($self->{'hash'}->{$a}) . ' union ' . Dumper($self->{'hash'}->{$b}) . ' is ' . Dumper($self->{'hash'}->{$id}) . "\n";
@@ -138,7 +150,7 @@ sub union {
 
 sub intersection {
 	my ($self, $a, $b) = @_;
-	my $id = new_id;
+	my $id = $self->new_id;
 	$self->{'hash'}->{$id} = $self->join_sets('i', $self->{'key'}, $self->{'hash'}->{$a}, $self->{'hash'}->{$b});
 	#use Data::Dumper;
 	#warn Dumper($self->{'hash'}->{$a}) . ' intersection ' . Dumper($self->{'hash'}->{$b}) . ' is ' . Dumper($self->{'hash'}->{$id}) . "\n";
@@ -147,7 +159,7 @@ sub intersection {
 
 sub negation {
 	my ($self, $a, $b) = @_;
-	my $id = new_id;
+	my $id = $self->new_id;
 	$self->{'hash'}->{$id} = $self->join_sets('n', $self->{'key'}, $self->{'hash'}->{$a}, $self->{'hash'}->{$b});
 	#use Data::Dumper;
 	#warn Dumper($self->{'hash'}->{$a}) . ' negation ' . Dumper($self->{'hash'}->{$b}) . ' is ' . Dumper($self->{'hash'}->{$id}) . "\n";
@@ -156,7 +168,7 @@ sub negation {
 
 sub difference {
 	my ($self, $a, $b) = @_;
-	my $id = new_id;
+	my $id = $self->new_id;
 	$self->{'hash'}->{$id} = $self->join_sets('d', $self->{'key'}, $self->{'hash'}->{$a}, $self->{'hash'}->{$b});
 	#use Data::Dumper;
 	#warn Dumper($self->{'hash'}->{$a}) . ' difference ' . Dumper($self->{'hash'}->{$b}) . ' is ' . Dumper($self->{'hash'}->{$id}) . "\n";
@@ -168,7 +180,8 @@ sub join_sets {
 	my (@intersection, @difference) = ();
 	my (%count, %objects) = ();
 	foreach my $e (@$a, @$b) {
-		#note: this assumes the lists @$a and @$b are made of UNIQUE items
+		#WARNING: this assumes the arrays @$a and @$b are made of UNIQUE items.
+		#Apache::Wyrd::Services::Index returns unique results from the search method.
 		$count{$e->{$index}}++;
 		$objects{$e->{$index}} = $e;
 	}
